==> azureml_dataprep-0.1.0-py3-none-any.whl: azureml/dataprep/api/dataprofile.py <==
# Copyright (c) Microsoft Corporation. All rights reserved.
from collections import namedtuple
from typing import Any, Dict, List
from .engineapi.api import EngineAPI
from .engineapi.typedefinitions import ActivityReference, DataField, ExecuteInspectorCommonArguments, FieldType, InspectorArguments
from .value import value_from_field

_MAX_ROW_COUNT = 2**31 - 1

ValueCountEntry = namedtuple('ValueCountEntry', ['value', 'count'])
HistogramBucket = namedtuple('HistogramBucket', ['lower_bound', 'upper_bound', 'count'])

class DataProfile:
    """
    A DataProfile collects summary statistics on the data produced by a Dataflow.

    :var columns: Profile information for each result column.
    :vartype columns: Dict[str, ColumnProfile]
    """

    def __init__(self, engine_api: EngineAPI, context: ActivityReference):
        self._engine_api = engine_api
        self._context = context

        table_inspector = self._engine_api.execute_inspector(ExecuteInspectorCommonArguments(
            context=self._context,
            inspector_arguments=InspectorArguments(inspector_type='Microsoft.DPrep.TableInspector', arguments={}),
            offset=0,
            row_count=_MAX_ROW_COUNT))
        column_names = [ column_definition.id for column_definition in table_inspector.column_definitions ]
        rows = table_inspector.rows_data.rows

        def values_for_column(fields):
            return {
                name: value_from_field(field)
                for name, field in zip(column_names, fields)
                if field is not None and field.type != FieldType.NULL and field.type != FieldType.ERROR
            }

        self.columns = { row[0].value: ColumnProfile(values_for_column(row)) for row in rows }

    def _repr_html_(self):
        """
        HTML representation for IPython.
        """
        try:
            import pandas as pd
        except ImportError:
            return None

        stats = [column_profile._get_stats() for column_profile in self.columns.values()]
        return pd.DataFrame(stats, index=self.columns.keys(), columns=ColumnProfile.STAT_COLUMNS).to_html()

    def __repr__(self):
        return '\n'.join(map(str, self.columns.values()))


class ColumnProfile:
    """
    A ColumnProfile collects summary statistics on a particular column of data produced by a Dataflow.

    :var name: Name of column
    :vartype name: str
    :var type: Type of values in column
    :vartype type: FieldType

    :var min: Minimum value
    :vartype min: Any
    :var max: Maximum value
    :vartype max: Any
    :var count: Count of rows
    :vartype count: int
    :var missing_count: Count of rows with a missing value
    :vartype missing_count: int
    :var error_count: Count of rows with an error value
    :vartype error_count: int

    :var lower_quartile: Estimated 25th-percentile value
    :vartype lower_quartile: double
    :var median: Estimated median value
    :vartype median: double
    :var upper_quartile: Estimated 75th-percentile value
    :vartype upper_quartile: double
    :var std: Standard deviation
    :vartype std: double
    :var mean: Mean
    :vartype mean: double
    """
    def __init__(self, values: Dict[str, Any]):
        self.name = values.get('Column')
        self.type = FieldType(values.get('type'))

        self.min = values.get('min')
        self.max = values.get('max')
        self.count = values.get('count')
        self.missing_count = values.get('num_missing')
        self.error_count = values.get('num_errors')

        self._prepare_value_counts(values.get('value_count'))

        if self._is_numeric:
            self.lower_quartile = values.get('25%')
            self.median = values.get('50%')
            self.upper_quartile = values.get('75%')
            self.std = values.get('std')
            self.mean = values.get('mean')
            self._prepare_histogram(values.get('histogram'))

    @property
    def _is_numeric(self):
        return self.type == FieldType.INTEGER or self.type == FieldType.DECIMAL

    def _prepare_value_counts(self, entries: List[Any]):
        if entries and len(entries) > 0:
            def process_entry(entry):
                value = value_from_field(DataField.from_pod(entry['value']))
                count = int(entry['count'])
                return ValueCountEntry(value, count)
            self.value_counts = [ process_entry(entry) for entry in entries ]

    def _prepare_histogram(self, fields: List[Dict[str, Any]]):
        if fields:
            values = (f['value'] for f in fields)
            def generate_buckets():
                last_position = None
                for position, count in zip(values, values):
                    if last_position is not None:
                        yield HistogramBucket(lower_bound=last_position, upper_bound=position, count=count)
                    last_position = position
            histogram = list(generate_buckets())
            if len(histogram) > 0:
                self.histogram = histogram

    def _get_stats(self):
        return [
            self.type, self.min, self.max, self.count, self.missing_count, self.error_count,
            self.lower_quartile if self._is_numeric else '',
            self.upper_quartile if self._is_numeric else '',
            self.std if self._is_numeric else '',
            self.mean if self._is_numeric else ''
        ]

    STAT_COLUMNS = [
        'Type', 'Min', 'Max', 'Count', 'Missing Count', 'Error Count',
        'Lower Quartile', 'Upper Quartile', 'Standard Deviation', 'Mean'
    ]

    def _repr_html_(self):
        """
        HTML representation for IPython.
        """
        try:
            import pandas as pd
        except ImportError:
            return None
        return pd.DataFrame(self._get_stats(), index=ColumnProfile.STAT_COLUMNS, columns=['Statistics']).to_html()

    def __repr__(self):
        result = """\
ColumnProfile
    name: {name}
    type: {type}

    min: {min}
    max: {max}
    count: {count}
    missing_count: {missing_count}
    error_count: {error_count}
""".format(**vars(self))

        if self._is_numeric:
            result += """\

    lower_quartile: {lower_quartile}
    median: {median}
    upper_quartile: {upper_quartile}
    std: {std}
    mean: {mean}
""".format(**vars(self))
        return result

==> azureml_dataprep-0.3.0-py3-none-any.whl: azureml/dataprep/api/dataprofile.py <==
# Copyright (c) Microsoft Corporation. All rights reserved.
from collections import namedtuple
from typing import Any, Dict, List
from .engineapi.api import EngineAPI
from .engineapi.typedefinitions import ActivityReference, DataField, ExecuteInspectorCommonArguments, FieldType, InspectorArguments
from .value import value_from_field

_MAX_ROW_COUNT = 2**31 - 1

ValueCountEntry = namedtuple('ValueCountEntry', ['value', 'count'])
HistogramBucket = namedtuple('HistogramBucket', ['lower_bound', 'upper_bound', 'count'])

class DataProfile:
    """
    A DataProfile collects summary statistics on the data produced by a Dataflow.

    :var columns: Profile information for each result column.
    :vartype columns: Dict[str, ColumnProfile]
    """

    def __init__(self, engine_api: EngineAPI, context: ActivityReference):
        self._engine_api = engine_api
        self._context = context

        table_inspector = self._engine_api.execute_inspector(ExecuteInspectorCommonArguments(
            context=self._context,
            inspector_arguments=InspectorArguments(inspector_type='Microsoft.DPrep.TableInspector', arguments={}),
            offset=0,
            row_count=_MAX_ROW_COUNT))
        column_names = [ column_definition.id for column_definition in table_inspector.column_definitions ]
        rows = table_inspector.rows_data.rows

        def values_for_column(fields):
            return {
                name: value_from_field(field)
                for name, field in zip(column_names, fields)
                if field is not None and field.type != FieldType.NULL and field.type != FieldType.ERROR
            }

        self.columns = { row[0].value: ColumnProfile(values_for_column(row)) for row in rows }

    def _repr_html_(self):
        """
        HTML representation for IPython.
        """
        try:
            import pandas as pd
        except ImportError:
            return None

        stats = [column_profile._get_stats() for column_profile in self.columns.values()]
        return pd.DataFrame(stats, index=self.columns.keys(), columns=ColumnProfile.STAT_COLUMNS).to_html()

    def __repr__(self):
        return '\n'.join(map(str, self.columns.values()))


class ColumnProfile:
    """
    A ColumnProfile collects summary statistics on a particular column of data produced by a Dataflow.

    :var name: Name of column
    :vartype name: str
    :var type: Type of values in column
    :vartype type: FieldType

    :var min: Minimum value
    :vartype min: Any
    :var max: Maximum value
    :vartype max: Any
    :var count: Count of rows
    :vartype count: int
    :var missing_count: Count of rows with a missing value
    :vartype missing_count: int
    :var error_count: Count of rows with an error value
    :vartype error_count: int

    :var lower_quartile: Estimated 25th-percentile value
    :vartype lower_quartile: double
    :var median: Estimated median value
    :vartype median: double
    :var upper_quartile: Estimated 75th-percentile value
    :vartype upper_quartile: double
    :var std: Standard deviation
    :vartype std: double
    :var mean: Mean
    :vartype mean: double

    :var value_counts: Counts of discrete values in the data; None if too many values.
    :vartype value_counts: List[ValueCountEntry]
    :var histogram: Histogram buckets showing the distribution of the data; None if data is non-numeric.
    :vartype histogram: List[HistogramBucket]
    """
    def __init__(self, values: Dict[str, Any]):
        self.name = values.get('Column')
        self.type = FieldType(values.get('type'))

        self.min = values.get('min')
        self.max = values.get('max')
        self.count = values.get('count')
        self.missing_count = values.get('num_missing')
        self.error_count = values.get('num_errors')

        self.value_counts = self._prepare_value_counts(values.get('value_count'))

        # Present for numeric columns only, will otherwise be None.
        self.lower_quartile = values.get('25%')
        self.median = values.get('50%')
        self.upper_quartile = values.get('75%')
        self.std = values.get('std')
        self.mean = values.get('mean')
        self.histogram = self._prepare_histogram(values.get('histogram'))

    @property
    def _is_numeric(self):
        return self.type == FieldType.INTEGER or self.type == FieldType.DECIMAL

    def _prepare_value_counts(self, entries: List[Any]):
        if entries and len(entries) > 0:
            def process_entry(entry):
                value = value_from_field(DataField.from_pod(entry['value']))
                count = int(entry['count'])
                return ValueCountEntry(value, count)
            return [ process_entry(entry) for entry in entries ]
        return None

    def _prepare_histogram(self, fields: List[Dict[str, Any]]):
        if fields:
            values = (f['value'] for f in fields)
            def generate_buckets():
                last_position = None
                for position, count in zip(values, values):
                    if last_position is not None:
                        yield HistogramBucket(lower_bound=last_position, upper_bound=position, count=count)
                    last_position = position
            histogram = list(generate_buckets())
            if len(histogram) > 0:
                return histogram
        return None

    def _get_stats(self):
        return [
            self.type, self.min, self.max, self.count, self.missing_count, self.error_count,
            self.lower_quartile if self._is_numeric else '',
            self.median if self._is_numeric else '',
            self.upper_quartile if self._is_numeric else '',
            self.std if self._is_numeric else '',
            self.mean if self._is_numeric else ''
        ]

    STAT_COLUMNS = [
        'Type', 'Min', 'Max', 'Count', 'Missing Count', 'Error Count',
        'Lower Quartile', 'Median', 'Upper Quartile', 'Standard Deviation', 'Mean'
    ]

    def _repr_html_(self):
        """
        HTML representation for IPython.
        """
        try:
            import pandas as pd
        except ImportError:
            return None
        return pd.DataFrame(self._get_stats(), index=ColumnProfile.STAT_COLUMNS, columns=['Statistics']).to_html()

    def __repr__(self):
        result = """\
ColumnProfile
    name: {name}
    type: {type}

    min: {min}
    max: {max}
    count: {count}
    missing_count: {missing_count}
    error_count: {error_count}
""".format(**vars(self))

        if self._is_numeric:
            result += """\

    lower_quartile: {lower_quartile}
    median: {median}
    upper_quartile: {upper_quartile}
    std: {std}
    mean: {mean}
""".format(**vars(self))
        return result
